Opendrive parser
================

## Description

This library is a set of C++ classes that represent the [Opendrive description](https://www.asam.net/standards/detail/opendrive/) of a road.

The main object is the [COpendriveRoad](opendrive_road.md) which describes the whole road as a set of [COpendriveRoadSegment](opendrive_road_segment.md) objects.

The [COpendriveRoadSegment](opendrive_road_segment.md) describes a uniform segment of the road (that is, one in which the number of lanes at each side remains the same) and includes the connectivity with other road segments. Each of the lanes on a road segment is descrived as a [COpendriveLane](opendrive_lane.md) object. The road segments also includes information on the traffic signals [COpendriveSignals](opendrive_signals.md) and obstacles [COpendriveObjects](opendrive_objects.md) belonging to it.

The [COpendriveLane](opendrive_lane.md) object describes a single lane of a road segment and its connectivity with any previous or next lanes on different segments, or adjacent lanes on the same side in the same segment. Each lane is descrived as a set of [COpendriveRoadNode](opendrive_road_node.md) objects.

The [COpendriveRoadNode](opendrive_road_node.md) objects are placed at the locations where the lane starts or places where the geometry changes, and include the connectivity with other adjacent road nodes. Each Link with other road nodes is unidirectional and is described by a [COpendriveLink](opendrive_link.md) object.

The [COpendriveLink](opendrive_link.md) objects include all the geometry information associated with the connection. In this case the geometry is describes as a [G2 spline](../doc/g2_spline.md).

The road nodes and the links between them describe the road as a directed graph.

### Class structure

The next diagram shows the basic class structure.

<img src="images/class_structure.png" alt="Class structure of the opendrive library" width="1058" height="595">

In this diagram, arrows indicate direct use of one class by another one, which coincides with the previous description. However, the COpendriveRoad class is the only one than handles (creates and destroys) COpendriveRoadSegment, OpendriveLane or OpendriveRoadNode objects (solid arrows), and the other classes use only references to those objects (dotted arrows). Lines with a triangle indicates inheritance.

## Common features

Some features common to all these classes are:

* Set of common parameters to modify the road representation. All these parameters can only be modified through the main COpendriveRoad object, and can be modified at any time. Keep in mind however, that once the road description has been loaded and the internal geometry generated, the computational cost of changing these parameters can be high, so it is recommended to set these values before loading the road description. These parameters are:

    * **resolution**: this is the minimum distance between two consecutive points in any road geometry.
    * **scale**: this is a scale factor that is applied to scale up or down the road description withou actually modifying the internal geometry information.
    * **min_road_length**: this is the minimum length of a road segment to be included when the opendrive file is loaded. Any road segment shorter than this value will be ignored.

* The *<<* operator is overloaded to display human readable information about the road structure.

* The public API is intended to provide access to all the important information of each of the road elements, but without being able to modify them. 

## Example road

The road shown in the next figure will be used to complete the description of all classes with an example. The opendrive road descrption can be downloaded [here]().

<img src="images/original_road.png" alt="Opendrive road description used as an example" width="814" height="594">

The dotted lines represent the limits of adjacent road segments. Each road segment include its name and the opendrive segment identifier (in parenthesis). The name of the two road junctions are also displayed but the name of each junction road is omitted for clarity. The name of the road junctions follows the next format:

```
<junction_name>_<origin_road_id>_<origin_lane_id>_<target_road_id>_<target_lane_id>
```

## Important
This is a work in progess and may suffer some changes and updates, although the general 

## How to use it

* Create a COpendriveRoad object.
* Set the desired resolution, scale and minimum road length.
* Import the Opendrive description file using the *load()* function.
* Retrieve the desired information using the *get_\*()* public functions.

This library provides several examples to demonstrate its use in some simple cases:

* Load a road description and shown the road description on screen ()
* Laad a road description and show the 